package com.github.jfcloud.gene.lis.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.db.Entity;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.jfcloud.gene.lis.config.LisDb;
import com.github.jfcloud.gene.lis.entity.HtScienceApply;
import com.github.jfcloud.gene.lis.entity.HtScienceApplyItem;
import com.github.jfcloud.gene.sample.entity.SampleCheckOrder;
import com.github.jfcloud.gene.sample.entity.SampleInfo;
import com.github.jfcloud.gene.sample.enums.ScientificSampleTypeEnum;
import com.github.jfcloud.gene.sample.mapper.SampleCheckOrderMapper;
import com.github.jfcloud.gene.sample.mapper.SampleInfoMapper;
import com.github.jfcloud.gene.sample.service.SampleAnimalService;
import com.github.jfcloud.gene.sample.service.SampleCheckItemSerumService;
import com.github.jfcloud.gene.sample.vo.CheckItemSerumVo;
import com.github.jfcloud.gene.sample.vo.SampleAnimalVo;
import com.github.jfcloud.gene.sample.vo.SampleCheckSerumVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class LisSyncService {

    private final LisDb lisDb;
    private final SampleInfoMapper sampleInfoMapper;
    private final SampleCheckOrderMapper checkOrderMapper;
    private final SampleAnimalService sampleAnimalService;
    private final SampleCheckItemSerumService sampleCheckItemSerumService;

    /**
     * 同步LIS
     */
    public void sync(Long sampleId) {
        SampleAnimalVo animalDetail = sampleAnimalService.getDetail(sampleId);
        Assert.notNull(animalDetail, "同步LIS失败，动物样本检测不存在");
        Assert.isTrue(StrUtil.isNotBlank(animalDetail.getScientificSampleType()) &&
                animalDetail.getScientificSampleType().contains(ScientificSampleTypeEnum.serum.getDescription()), "同步LIS失败，只有包含血清样本送检才可以同步");
        SampleCheckSerumVo serum = animalDetail.getOrder().getSerum();

        SampleCheckOrder checkOrder = checkOrderMapper.selectById(serum.getId());
        JSONObject extObj = new JSONObject();
        if (StrUtil.isNotBlank(checkOrder.getExtData())) {
            extObj = JSON.parseObject(checkOrder.getExtData());
            Assert.isFalse(extObj.containsKey("lisId"), "同步LIS失败，已同步过");
        }

        log.info("开始同步lis数据库 sample={}", sampleId);

        //HT_SCIENCE_APPLY主键非自增，为年份2位+月份2位+日期2位+483+7位序列号，为了避免冲突，这里使用480
        SampleInfo sampleInfo = sampleInfoMapper.selectById(sampleId);
        String idPrefix = sampleInfo.getApplyTime().format(DateTimeFormatter.ofPattern("yyMMdd")) + "480";
        long firstId = Long.parseLong(String.format("%s%07d", idPrefix, 1));
        long itemFirstId = Long.parseLong(String.valueOf(firstId).replace("480", "481"));
        //编号起始与结尾
        int beginNo = 1;
        int endNo = 1;
        //查询当前最大的id与编号（编号格式HKYBKY20250326-14-15）
        String sql = String.format("select top 1 id, SampleNo from dbo.HT_SCIENCE_APPLY where id like '%s%%' order by id desc ",  idPrefix);
        Entity idEntity = lisDb.queryOne(sql);
        if (idEntity != null) {
            firstId = idEntity.getLong("id") + 1;
            String sampleNo = idEntity.getStr("SampleNo");
            String[] split = sampleNo.split("-");
            beginNo = Integer.parseInt(split[2]) + 1;
        }
        log.info("同步lis数据库 HT_SCIENCE_APPLY起始id={} HT_SCIENCE_APPLY_ITEM起始id={} ", firstId, itemFirstId);

        //查询当天的样本编号
        String noPrefix = "HKYBKY" + sampleInfo.getApplyTime().format(DateTimeFormatter.ofPattern("yyyyMMdd"));

        //查询科室表HT_INFO_DEP对应的机构
        List<Entity> depEntities = lisDb.query("select id, name, hsp_id from dbo.HT_INFO_DEP");
        Map<Long, Entity> depMap = depEntities.stream().collect(Collectors.toMap(e -> e.getLong("id"), Function.identity()));

        //根据昵称查询用户id
        Entity applyUser = lisDb.queryOne("select top 1 user_id from dbo.HT_INFO_USER_EXTEND where name = '" + serum.getApplyUserName() + "' order by user_id desc");

        //同步LIS
        List<HtScienceApply> applyList = new ArrayList<>();
        List<HtScienceApplyItem> applyItemList = new ArrayList<>();
        for (CheckItemSerumVo item : serum.getItems()) {
            HtScienceApply scienceApply = new HtScienceApply();
            scienceApply.setId(firstId++);
            if (serum.getSendOrgId() != null) {
                scienceApply.setHspId(depMap.get(serum.getSendOrgId()).getStr("hsp_id"));
            }
            scienceApply.setPetType(item.getLisPetTypeId());
            scienceApply.setEmail(serum.getApplyUserEmail());
            scienceApply.setPhone(serum.getApplyUserMobile());
            scienceApply.setCreateName(serum.getApplyUserName());
            scienceApply.setCreateId(applyUser.getStr("user_id"));
            scienceApply.setCreateTime(checkOrder.getCreateTime());
            scienceApply.setSex(item.getLisSexId());
            scienceApply.setAge(Integer.parseInt(item.getAge()));
            scienceApply.setFastFlag(Integer.parseInt(item.getQuick()));
            scienceApply.setApplyDep(serum.getSendOrgId().toString());
            scienceApply.setApplyDocId(serum.getSendUserId().toString());
            scienceApply.setApplyDoc(serum.getSendUserName());
            scienceApply.setApplyTime(serum.getApplyDate());
            scienceApply.setDeliverOper(serum.getSendUserName());
            scienceApply.setDeliverOperId(serum.getSendUserId().toString());
            scienceApply.setDeliverTime(serum.getApplyDate());
            scienceApply.setSampleCount(Integer.parseInt(item.getCodeSuffix()));
            scienceApply.setPreText(item.getCodePrefix());
            scienceApply.setBeginNo(String.format("%0" + item.getCodeSuffix().length() +"d", 1));

            endNo = beginNo + scienceApply.getSampleCount() - 1;
            scienceApply.setSampleNo(noPrefix + "-" + beginNo + "-" + endNo);
            applyList.add(scienceApply);

            //检测指标
            if (StrUtil.isNotBlank(item.getTarget())) {
                List<Long> itemIds = JSON.parseArray(item.getTarget(), Long.class);
                for (Long itemId : itemIds) {
                    HtScienceApplyItem applyItem = new HtScienceApplyItem();
                    applyItem.setId(itemFirstId++);
                    applyItem.setApplyId(scienceApply.getId());
                    applyItem.setItemId(itemId);
                    applyItemList.add(applyItem);
                }
            }
        }

        Assert.notEmpty(applyList, "同步LIS失败，无血清送检单");
        Assert.notEmpty(applyItemList, "同步LIS失败，无检测指标");
        JSONObject finalExtObj = extObj;
        lisDb.execute(db -> {
            try {
                List<Entity> applyEntities = applyList.stream().map(x -> Entity.create("dbo.HT_SCIENCE_APPLY").parseBean(x)).collect(Collectors.toList());
                db.insert(applyEntities);
                List<Entity> applyItemEntities = applyItemList.stream().map(x -> Entity.create("dbo.HT_SCIENCE_APPLY_ITEM").parseBean(x)).collect(Collectors.toList());
                db.insert(applyItemEntities);

                //保存id
                List<Long> applyIdList = applyList.stream().map(HtScienceApply::getId).collect(Collectors.toList());
                finalExtObj.put("lisId", applyIdList);
                SampleCheckOrder co = new SampleCheckOrder();
                co.setId(checkOrder.getId());
                co.setExtData(finalExtObj.toString());
                co.updateById();

                log.info("同步lis数据库完成 sampleId={} ", sampleId);
            } catch (Exception e) {
                log.error("同步LIS失败", e);
            }
        });
    }

    /**
     * 取消同步LIS
     */
    public void cancelSyncLis(Long sampleId) {
        SampleAnimalVo animalDetail = sampleAnimalService.getDetail(sampleId);
        Assert.notNull(animalDetail, "取消同步LIS失败，动物样本检测不存在");
        SampleCheckSerumVo serum = animalDetail.getOrder().getSerum();

        SampleCheckOrder checkOrder = checkOrderMapper.selectById(serum.getId());
        Assert.notEmpty(checkOrder.getExtData(), "取消同步LIS失败，未同步");
        JSONObject extObj = JSON.parseObject(checkOrder.getExtData());
        Assert.isTrue(extObj.containsKey("lisId"), "取消同步LIS失败，未同步");

        log.info("开始取消同步lis数据库 sample={}", sampleId);
        JSONArray array = extObj.getJSONArray("lisId");
        List<Long> applyIdList = JSON.parseArray(array.toJSONString(), Long.class);

        lisDb.execute(db -> {
            try {
                String applyIdJoin = CollUtil.join(applyIdList, ",");
                db.execute("delete from dbo.HT_SCIENCE_APPLY where id in (" + applyIdJoin + ")");
                db.execute("delete from dbo.HT_SCIENCE_APPLY_ITEM where apply_id in (" + applyIdJoin + ")");
                log.info("取消同步lis数据库完成 sampleId={} ", sampleId);
            } catch (Exception e) {
                log.error("取消同步LIS失败", e);
            }
        });
    }
}
