package com.github.jfcloud.gene.util;

import cn.hutool.core.collection.CollUtil;
import lombok.Data;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.docx4j.XmlUtils;
import org.docx4j.dml.spreadsheetdrawing.CTDrawing;
import org.docx4j.openpackaging.packages.SpreadsheetMLPackage;
import org.docx4j.openpackaging.parts.DrawingML.Drawing;
import org.docx4j.openpackaging.parts.PartName;
import org.docx4j.openpackaging.parts.SpreadsheetML.WorksheetPart;
import org.docx4j.openpackaging.parts.WordprocessingML.BinaryPartAbstractImage;
import org.springframework.stereotype.Service;

import java.util.Comparator;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@Slf4j
@RequiredArgsConstructor
@Service
public class ExcelImageService {

    private final WordDataService wordDataService;


    /**
     * 给Excel添加图片
     *
     * @param pkg        excel包
     * @param sheetIndex sheet下标，从0开始
     * @param imageList  图片列表
     */
    @SneakyThrows
    public void addImageDrawingPart(SpreadsheetMLPackage pkg, int sheetIndex, List<ExcelImage> imageList) {
        if (CollUtil.isEmpty(imageList)) {
            log.info("Excel单元格 插入图片失败：图片不存在");
            return;
        }

        List<String> imgUrls = imageList.stream().map(ExcelImage::getImageUrl).distinct().collect(Collectors.toList());
        for (String imageUrl : imgUrls) {
            byte[] bytes = wordDataService.getBytes(imageUrl);
            imageList.stream().filter(x -> x.getImageUrl().equals(imageUrl)).forEach(x -> x.setBytes(bytes));
        }

        //在指定的sheet添加图片
        WorksheetPart worksheet = pkg.getWorkbookPart().getWorksheet(sheetIndex);

        // 签名图片的xml
        String signDrawingName = "/xl/drawings/drawing_sign.xml";
        Drawing part = new Drawing(new PartName(signDrawingName));

        // Add anchor XML to worksheet
        org.xlsx4j.sml.CTDrawing smlCtDrawing = org.xlsx4j.jaxb.Context.getsmlObjectFactory().createCTDrawing();
        smlCtDrawing.setId(worksheet.addTargetPart(part).getId());
        worksheet.getJaxbElement().setDrawing(smlCtDrawing);

        StringBuilder sb = new StringBuilder();
        sb.append("<xdr:wsDr xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\" xmlns:a14=\"http://schemas.microsoft.com/office/drawing/2010/main\" xmlns:xdr=\"http://schemas.openxmlformats.org/drawingml/2006/spreadsheetDrawing\" xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\">");

        imageList.sort(Comparator.comparing(ExcelImage::getLocation));
        for (int i = 1; i <= imageList.size(); i++) {
            ExcelImage excelImage = imageList.get(i - 1);

            // 创建图片
            BinaryPartAbstractImage imagePart = BinaryPartAbstractImage.createImagePart(pkg, part, excelImage.getBytes());
            String imageRelID = imagePart.getSourceRelationships().get(0).getId();
            log.info("Excel单元格[{}] relId={} 插入图片：{}", excelImage.getLocation(), imageRelID, excelImage.getImageUrl());

            String openXML = "<xdr:twoCellAnchor editAs=\"oneCell\">"
                    + "<xdr:from>"
                    + "<xdr:col>" + excelImage.col + "</xdr:col>"
                    + "<xdr:colOff>0</xdr:colOff>"
                    + "<xdr:row>" + excelImage.row + "</xdr:row>"
                    + "<xdr:rowOff>0</xdr:rowOff>"
                    + "</xdr:from>"
                    + "<xdr:to>"
                    + "<xdr:col>" + excelImage.col + "</xdr:col>"
                    + "<xdr:colOff>603250</xdr:colOff>"
                    + "<xdr:row>" + excelImage.row + "</xdr:row>"
                    + "<xdr:rowOff>796500</xdr:rowOff>"
                    + "</xdr:to>"
                    + "<xdr:pic>"
                    + "<xdr:nvPicPr>"
                    + "<xdr:cNvPr id=\"" + i + "\" name=\"Picture " + i + "\"/>"
                    + "<xdr:cNvPicPr>"
                    + "<a:picLocks noChangeAspect=\"1\"/>"
                    + "</xdr:cNvPicPr>"
                    + "</xdr:nvPicPr>"
                    + "<xdr:blipFill>"
                    + "<a:blip r:embed=\"" + imageRelID + "\">"
                    + "<a:extLst>"
                    + "<a:ext uri=\"{" + UUID.randomUUID() + "}\">"
                    + "<a14:useLocalDpi val=\"0\"/>"
                    + "</a:ext>"
                    + "</a:extLst>"
                    + "</a:blip>"
                    + "<a:stretch>"
                    + "<a:fillRect/>"
                    + "</a:stretch>"
                    + "</xdr:blipFill>"
                    + "<xdr:spPr bwMode=\"auto\">"
                    + "<a:xfrm>"
                    + "<a:off x=\"3794124\" y=\"9294813\"/>"
                    + "<a:ext cx=\"603251\" cy=\"357215\"/>"
                    + "</a:xfrm>"
                    + "<a:prstGeom prst=\"rect\">"
                    + "<a:avLst/>"
                    + "</a:prstGeom>"
                    + "</xdr:spPr>"
                    + "</xdr:pic>"
                    + "<xdr:clientData/>"
                    + "</xdr:twoCellAnchor>";

            sb.append(openXML);
        }

        sb.append("</xdr:wsDr>");
        CTDrawing dmlCtDrawing = (CTDrawing) XmlUtils.unwrap(XmlUtils.unmarshalString(sb.toString()));
        part.setJaxbElement(dmlCtDrawing);
    }


    /**
     * excel图片
     */
    @Data
    public static class ExcelImage {

        /**
         * 图片链接
         */
        private String imageUrl;

        /**
         * 图片字节数组
         */
        private byte[] bytes;

        /**
         * 图片位置，如B23
         */
        private String location;

        private int col;

        private int row;

        public void setLocation(String location) {
            this.location = location;
            this.col = location.charAt(0) - 'A';
            this.row = Integer.parseInt(location.substring(1)) - 1;
        }

        public ExcelImage(String imageUrl, String location) {
            this.imageUrl = imageUrl;
            setLocation(location);
        }
    }
}
